from __future__ import print_function

import pyzabbix
import sys
import zm.utils
import zm.zutils


PRIORITY = {
    "NOTCLASSIFIED": "0",
    "INFORMATION": "1",
    "WARNING": "2",
    "AVERAGE": "3",
    "HIGH": "4",
    "DISASTER": "5",
}


@zm.utils.login
def list(conn,
         host=None,
         priority=None,
         problematic=True,
         monitored=True,
         unacknowledged=True):
    params = {}
    flags = {}

    flags["selectHosts"] = "extend"

    if host:
        try:
            int(host)
            params["hostids"] = host
        except ValueError:
            params["host"] = host

    if priority:
        try:
            params["priority"] = PRIORITY[priority]
        except IndexError:
            try:
                int(priority)
                params["priority"] = priority
            except ValueError:
                pass

    if problematic:
        params["value"] = "1"

    if monitored:
        flags["monitored"] = "extend"

    if unacknowledged:
        flags["withLastEventUnacknowledged"] = "extend"


    l = []
    for trigger in zm.zutils.get(conn,
                                 type="trigger",
                                 params=params,
                                 # 'extend' is not recommended since
                                 # can hit memory allocation on server
                                 output=["triggerid",
                                         "description",
                                         "value",
                                         "hosts",
                                         "priority"],
                                 **flags):
        trigger.update({"host_at": trigger["hosts"][0]["host"]})
        l.append(trigger)

    return l



import sys
import zm.commands
import zm.config

from oslo.config import cfg


opts = [
    cfg.StrOpt("username",
               help="Zabbix login."),
    cfg.StrOpt("password",
               help="Zabbix credentials."),
    cfg.StrOpt("url",
               help="Zabbix endpoint."),
]


CONF = cfg.CONF
CONF.register_cli_opts(opts)


def main():
    zm.config.parse_args(sys.argv)
    zm.commands.CommandManager().execute()


if __name__ == "__main__":
    main()



from __future__ import print_function

import prettytable
import pyzabbix
import requests
import sys

from oslo.config import cfg

CONF = cfg.CONF


def login(func):
    def _login(*args, **kwargs):
        try:
            s = requests.Session()
            s.auth = (CONF.username,
                      CONF.password)
            s.verify = False

            conn = pyzabbix.ZabbixAPI(CONF.url, s)
            #logger.debug("Login to Zabbix (user '%s')" % CONF.username)
            print("Login to Zabbix (user '%s')" % CONF.username)
            conn.login(CONF.username, CONF.password)
        except requests.exceptions.HTTPError:
            #logger.error("Zabbix authorization failed")
            print("Zabbix authorization failed")
            sys.exit(1)

        #logger.debug("Successfully connected to Zabbix API v%s" % conn.api_version())
        return func(conn, *args, **kwargs)
    return _login


class TableOutput(object):
    def __init__(self, l, output):
        self.l = l
        self.output = output

    def __str__(self):
        l = []
        for d in self.l:
            l.append([d[i] for i in self.output.keys()])

        return self.print_table(self.output.values(), sorted(l))

    def print_table(self, columns, row_content, padding_width=1):
        """
        Prints a prettytable for the arguments received.
            columns; list of column names
            row_content; list of lists containing column values
        """
        x = prettytable.PrettyTable(columns)
        x.padding_width = padding_width
        for row in row_content:
            x.add_row(row)
        return str(x)



import pyzabbix
import zm.utils
import zm.zutils

@zm.utils.login
def list(conn, host=None, group=None):
    if host:
        return zm.zutils.get(conn,
                             type="host",
                             params={"name": host},
                             output="extend",
                             **{"selectParentTemplates": "extend"})[0]["parentTemplates"]
    elif group:
        return zm.zutils.get(conn,
                             type="hostgroup",
                             params={"name": group},
                             output="extend",
                             selectTemplates="extend")[0]["templates"]
    else:
        return zm.zutils.get(conn,
                             type="template",
                             output="extend")




def get(conn, type, params={}, output="extend", **kwargs):
    types = {
        "host": conn.host.get,
        "hostgroup": conn.hostgroup.get,
        "item": conn.item.get,
        "template": conn.template.get,
        "trigger": conn.trigger.get,
        "usermacro": conn.usermacro.get,
    }

    return types[type](filter=params, output=output, **kwargs)



from __future__ import print_function

import zm.utils
import zm.zutils


@zm.utils.login
def list(conn):
    return zm.zutils.get(conn,
                         type="hostgroup",
                         output="extend")



import pyzabbix
import zm.utils
import zm.zutils


@zm.utils.login
def list(conn):
    # FIXME didn't find a way to distinguish between hostids that refer
    # to templateid or indeed to hostid
    l = []
    for macro in zm.zutils.get(conn,
                               type="usermacro",
                               output="extend",
                               **{"globalmacroids": "extend"}):
        # Set scope
        if zm.zutils.get(conn,
                         type="host",
                         params={"hostid": macro["hostid"]},
                         output=["hostid"]):
            scope = "Host level"
            macro.update({"scope": "Host level"})
        elif zm.zutils.get(conn,
                           type="template",
                           params={"hostid": macro["hostid"]},
                           output=["templateid"]):
            scope = "Template level"
            macro.update({"scope": "Template level"})
        else:
            raise NotImplementedError("ID '%s' not matched neither as template or host")

        l.append(macro)

    return l


@zm.utils.login
def update(conn, id, value):
    try:
        conn.usermacro.update(hostmacroid=id,
                              value=value)
        #logging.debug("User macro ID %s successfully updated. New value: %s" % (id, value))
    except pyzabbix.ZabbixAPIException, e:
        #logger.debug(e)
        raise pyzabbix.ZabbixAPIException(e)



from __future__ import print_function

import sys
import zm.exception
import zm.hostgroup
import zm.item
import zm.macro
import zm.template
import zm.trigger
import zm.utils

from oslo.config import cfg
from zm.utils import TableOutput


CONF = cfg.CONF


def add_command_parsers(subparsers):
    CommandHostgroupList(subparsers)
    CommandItemList(subparsers)
    CommandItemEnable(subparsers)
    CommandItemDisable(subparsers)
    CommandMacroList(subparsers)
    CommandMacroUpdate(subparsers)
    CommandTemplateList(subparsers)
    CommandTriggerList(subparsers)


command_opt = cfg.SubCommandOpt('command',
                                title='Commands',
                                help='Show available commands.',
                                handler=add_command_parsers)

CONF.register_cli_opt(command_opt)


class Command(object):
    def __init__(self, parser, name, cmd_help):
        self.name = name
        self.cmd_help = cmd_help
        self.parser = parser.add_parser(name, help=cmd_help)
        self.parser.set_defaults(func=self.run)

    def run(self):
        raise NotImplementedError("Method must me overriden on subclass")


class CommandHostgroupList(Command):
    def __init__(self, parser, name="hostgroup-list",
                 cmd_help="List hostgroups."):
        super(CommandHostgroupList, self).__init__(parser, name, cmd_help)

        self.output = {"groupid": "ID", "name": "Name"}

    def run(self):
        print(TableOutput(zm.hostgroup.list(),
                          output=self.output))


class CommandItemList(Command):
    def __init__(self, parser, name="item-list",
                 cmd_help="List host items."):
        super(CommandItemList, self).__init__(parser, name, cmd_help)

        self.parser.add_argument("host",
                                 metavar="ID/HOSTNAME",
                                 help="Zabbix hostname or ID.")

        self.output = {"itemid": "ID", "name": "Name", "status": "Value" }

    def run(self):
        print(TableOutput(zm.item.list(CONF.command.host),
                          output=self.output))


class CommandItemEnable(Command):
    def __init__(self, parser, name="item-enable",
                 cmd_help="Enable host items."):
        super(CommandItemEnable, self).__init__(parser, name, cmd_help)

        self.parser.add_argument("id",
                                 nargs="+",
                                 metavar="ID/NAME",
                                 help="Zabbix item name or ID.")

        self.parser.add_argument("--host",
                                 metavar="ID/HOSTNAME",
                                 help="Filter by Zabbix hostname or ID.")

        self.parser.add_argument("--hostgroup",
                                 metavar="ID/HOSTGROUP",
                                 help="Filter by Zabbix hostgroup name or ID.")

    def run(self):
        zm.item.update(CONF.command.id,
                       0,
                       CONF.command.host,
                       CONF.command.hostgroup)


class CommandItemDisable(Command):
    def __init__(self, parser, name="item-disable",
                 cmd_help="Disable host items."):
        super(CommandItemDisable, self).__init__(parser, name, cmd_help)

        self.parser.add_argument("id",
                                 nargs="+",
                                 metavar="ID",
                                 help="Zabbix item name or ID.")

        self.parser.add_argument("--host",
                                 metavar="HOST",
                                 help="Filter by Zabbix host.")

        self.parser.add_argument("--hostgroup",
                                 metavar="HOSTGROUP",
                                 help="Filter by Zabbix hostgroup.")

    def run(self):
        zm.item.update(CONF.command.id,
                       1,
                       CONF.command.host,
                       CONF.command.hostgroup)


class CommandMacroList(Command):
    def __init__(self, parser, name="macro-list",
                 cmd_help="List user macros."):
        super(CommandMacroList, self).__init__(parser, name, cmd_help)

        self.output = {"scope": "Scope",
                       "hostmacroid": "ID",
                       "macro": "Name",
                       "value": "Value"}

    def run(self):
        print(TableOutput(zm.macro.list(),
                          output=self.output))


class CommandMacroUpdate(Command):
    def __init__(self, parser, name="macro-update",
                 cmd_help="Update user macros."):
        super(CommandMacroUpdate, self).__init__(parser, name, cmd_help)

        self.parser.add_argument("id",
                                 metavar="ID",
                                 help="Zabbix macro ID.")

        self.parser.add_argument("value",
                                 metavar="VALUE",
                                 help="Zabbix value.")

    def run(self):
        zm.macro.update(CONF.command.id,
                        CONF.command.value)


class CommandTemplateList(Command):
    def __init__(self, parser, name="template-list",
                 cmd_help="List templates."):
        super(CommandTemplateList, self).__init__(parser, name, cmd_help)

        self.parser.add_argument("--host",
                                 metavar="HOST",
                                 help="Zabbix host name.")

        self.parser.add_argument("--group",
                                 metavar="HOSTGROUP",
                                 help="Zabbix hosgroup name.")

        self.output = {"templateid": "ID",
                       "name": "Name",
                       "status": "Status"}

    def run(self):
        print(TableOutput(zm.template.list(CONF.command.host,
                                           CONF.command.group),
                          output=self.output))


class CommandTriggerList(Command):
    def __init__(self, parser, name="trigger-list",
                 cmd_help="List host triggers."):
        super(CommandTriggerList, self).__init__(parser, name, cmd_help)

        self.parser.add_argument("--host",
                                 metavar="ID/HOSTNAME",
                                 help="Zabbix hostname or ID.")

        self.parser.add_argument("--priority",
                                 "-p",
                                 metavar="[DISASTER|HIGH|AVERAGE|WARNING|INFORMATION|NOTCLASSIFIED]",
                                 help="Trigger priority or ID.")

        self.parser.add_argument("--problematic",
                                 action="store_true",
                                 help="Show only triggers in a problem state.")

        self.parser.add_argument("--monitored",
                                 action="store_true",
                                 help="Show only triggers mapped to enabled items.")

        self.parser.add_argument("--unacknowledged",
                                 action="store_true",
                                 help="Show only unacknowledged triggers.")

        self.output = {"triggerid": "ID",
                       "description": "Description",
                       "host_at": "Host name",
                       "value": "Status",
                       "priority": "Priority"}

    def run(self):
        print(TableOutput(zm.trigger.list(CONF.command.host,
                                          priority=CONF.command.priority,
                                          problematic=CONF.command.problematic,
                                          monitored=CONF.command.monitored,
                                          unacknowledged=CONF.command.unacknowledged),
                          self.output))


class CommandManager(object):
    def execute(self):
        try:
            CONF.command.func()
        except zm.exception.ZMException as e:
            print("ERROR: %s" % e, file=sys.stderr)
            sys.exit(1)
        except KeyboardInterrupt:
            print ("\nExiting...", sys.stderr)
            sys.exit(0)



from __future__ import print_function

import pyzabbix
import sys
import zm.utils
import zm.zutils


ITEM_STATUS = {
    "0": "Enabled",
    "1": "Disabled",
    "3": "Not supported",
}

@zm.utils.login
def list(conn, host):
    params = {}

    try:
        int(host)
        params["hostids"] = host
    except ValueError:
        params["host"] = host

    return zm.zutils.get(conn,
                         type="item",
                         params=params,
                         value=host,
                         output="extend")


@zm.utils.login
def update(conn, id, status, host=None, hostgroup=None):
    items = []
    for narg in id:
        try:
            int(narg)
            parameter = "itemid"
        except ValueError:
            parameter = "name"

        params = {parameter: narg}
        print(params)

        items.extend(zm.zutils.get(conn,
                                   type="item",
                                   params={parameter: narg},
                                   output=["status", "name"],
                                   **{"selectHosts": ["name"]}))

    h_parameter = None
    if host:
        try:
            int(host)
            h_parameter = "hostids"
        except ValueError:
            h_parameter = "host"
    hg_parameter = None
    if hostgroup:
        try:
            int(hostgroup)
            hg_parameter = "groupids"
        except ValueError:
            hg_parameter = "group"

    if host or hostgroup:
        hitems = zm.zutils.get(conn,
                              type="item",
                              output=["status", "name"],
                              **{"filter": {"name": id,
                                            h_parameter : host,
                                            hg_parameter: hostgroup},
                                 "selectHosts": ["name"]})

        l_items = [i["itemid"] for i in items]
        for hi in hitems:
            if not hi["itemid"] in l_items:
                items.append(hi)

    try:
        for item in items:
            conn.item.update(itemid=item["itemid"], status=str(status))
            print("Item <%s (%s)> %s on host %s" % (item["name"],
                                                    item["itemid"],
                                                    ITEM_STATUS[str(status)].lower(),
                                                    item["hosts"][0]["name"]))
        if not items:
            print("No matching item found")
    except pyzabbix.ZabbixAPIException, e:
        if e.args[1] == -32602:
            print("Permission denied: failed to enable item.", file=sys.stderr)


